# Generated by Django 2.2.13 on 2021-03-04 12:01

from django.db import migrations


class Migration(migrations.Migration):
    dependencies = [
        ("partner", "0006_auto_20200724_0909"),
        ("partner", "0006_auto_20190612_0844"),
    ]

    operations = []
