from oscar.apps.partner import availability as oscar


class StockRequired(oscar.StockRequired):
    # pylint: disable=unused-argument
    def __init__(self, product, stockrecord):
        super().__init__(stockrecord.net_stock_level)
