from ocyan.plugin.oscar_partner.partner import abstract_models as ocyan


# pylint: disable=attribute-defined-outside-init
class AbstractStockRecord(ocyan.AbstractStockRecord):
    @property
    def price_excl_tax(self):
        return self.price

    @price_excl_tax.setter
    def price_excl_tax(self, value):
        self.price = value

    @property
    def price_incl_tax(self):
        return self.price

    @price_incl_tax.setter
    def price_incl_tax(self, value):
        self.price = value

    class Meta(ocyan.AbstractStockRecord.Meta):
        abstract = True
