from django.db import models

from wagtail.contrib.settings.models import BaseGenericSetting, register_setting


@register_setting
class LocalEUVatSettings(BaseGenericSetting):
    vat_threshold_exceeded = models.BooleanField(default=False)
