# -*- coding: utf-8 -*-
import doctest

import ocyan.plugin.oscar_partner.partner.utils


def load_tests(loader, tests, ignore):  # pylint: disable=W0613
    tests.addTests(doctest.DocTestSuite(ocyan.plugin.oscar_partner.partner.utils))
    return tests


# Create your tests here.
