from decimal import Decimal as D

from django.conf import settings
from stdnum.eu.vat import MEMBER_STATES

EU_LOCAL_VAT_TRESHOLD = D("10000")

EU_APPLICABLE_COUNTRIES = [
    country.upper()
    for country in MEMBER_STATES
    if country.upper() != settings.OSCAR_VAT_COUNTRY_OF_ORIGIN_CODE
]


LOCAL_VAT_APPLICABLE_KEY = "is_local_vat_applicable"
