# Generated by Django 2.0.13 on 2019-06-12 08:44

from django.db import migrations, models


class Migration(migrations.Migration):
    dependencies = [
        ("partner", "0005_auto_20181115_1953"),
    ]

    operations = [
        migrations.AlterField(
            model_name="stockrecord",
            name="num_in_stock",
            field=models.IntegerField(
                blank=True, null=True, verbose_name="Number in stock"
            ),
        ),
    ]
