# -*- coding: utf-8 -*-
from django.apps import AppConfig


class OcyanPluginOscarPartnerAppConfig(AppConfig):
    name = "ocyan.plugin.oscar_partner"
    label = "oscar_partner"

    def ready(self):
        from . import signals  # pylint: disable=unused-import, unused-variable
