from django.core.management.base import BaseCommand

from oscar.core.loading import get_model

LocalEUVatSettings = get_model("oscar_partner", "LocalEUVatSettings")


class Command(BaseCommand):
    def handle(self, *args, **options):
        eu_settings = LocalEUVatSettings.load()
        eu_settings.vat_threshold_exceeded = False
        eu_settings.save()
