# -*- coding: utf-8 -*-
default_app_config = "ocyan.plugin.oscar_partner.apps.OcyanPluginOscarPartnerAppConfig"

__all__ = ["OcyanOscar3PartnerAppPluginConfig"]


def __getattr__(name):
    if name != "OcyanOscar3PartnerAppPluginConfig":
        raise AttributeError(f"module {__name__!r} has no attribute {name!r}")
    from .entrypoint import OcyanOscar3PartnerAppPluginConfig

    return OcyanOscar3PartnerAppPluginConfig
