# Generated by Django 4.2.11 on 2024-06-03 09:54

from django.db import migrations


class Migration(migrations.Migration):
    dependencies = [
        ("oscar_partner", "0001_initial"),
    ]

    operations = [
        migrations.RemoveField(
            model_name="localeuvatsettings",
            name="site",
        ),
    ]
